"""Runner that outputs the residuals of the linear solver with Gnuplot"""

from StepAnalyzedCommon import StepAnalyzedCommon
from BasicRunner import BasicRunner
from BasicWatcher import BasicWatcher

from PyFoam.LogAnalysis.BoundingLogAnalyzer import BoundingPlotLogAnalyzer
from PyFoam.LogAnalysis.RegExpLineAnalyzer import RegExpTimeLineLineAnalyzer
from PyFoam.Basics.GnuplotTimelines import GnuplotTimelines
from PyFoam.Basics.TimeLineCollection import signedMax

class GnuplotCommon(StepAnalyzedCommon):
    """Class that collects the Gnuplotting-Stuff for two other classes"""
    def __init__(self,fname,smallestFreq=0.,persist=None,splitThres=2048,plotLinear=True,plotCont=True,plotBound=True,plotIterations=False,plotCourant=False,plotExecution=False,customRegexp=None):
        """
        TODO: Docu
        """
        StepAnalyzedCommon.__init__(self,fname,BoundingPlotLogAnalyzer(),smallestFreq=smallestFreq)

        self.plot=None
        self.plotCont=None
        self.plotBound=None
        self.plotIter=None
        self.plotCourant=None
        self.plotExecution=None
        self.plotCustom=None
        
        if plotLinear:
            self.plot=GnuplotTimelines(self.getAnalyzer("PlotLinear").lines,persist=persist)
            self.getAnalyzer("PlotLinear").lines.setSplitting(splitThres=splitThres,splitFun=max)
        
            self.plot.set_string("logscale y")
            self.plot.title("Residuals")

        if plotCont:
            self.plotCont=GnuplotTimelines(self.getAnalyzer("PlotContinuity").lines,persist=persist,alternateAxis=["Global"])
            self.plotCont.set_string("ylabel \"Cumulative\"")
            self.plotCont.set_string("y2label \"Global\"")
            self.getAnalyzer("PlotContinuity").lines.setSplitting(splitThres=splitThres)

            self.plotCont.title("Continuity")
 
        if plotBound:
            self.plotBound=GnuplotTimelines(self.getAnalyzer("PlotBounding").lines,persist=persist)
            self.getAnalyzer("PlotBounding").lines.setSplitting(splitThres=splitThres,splitFun=signedMax)
            self.plotBound.title("Bounded variables")

        if plotIterations:
            self.plotIter=GnuplotTimelines(self.getAnalyzer("PlotIterations").lines,persist=persist,with="steps")
            self.getAnalyzer("PlotIterations").lines.setSplitting(splitThres=splitThres)

            self.plotIter.title("Iterations")

        if plotCourant:
            self.plotCourant=GnuplotTimelines(self.getAnalyzer("PlotCourant").lines,persist=persist)
            self.getAnalyzer("PlotCourant").lines.setSplitting(splitThres=splitThres)

            self.plotCourant.title("Courant")
 
        if plotExecution:
            self.plotExecution=GnuplotTimelines(self.getAnalyzer("PlotExecution").lines,persist=persist,with="steps")
            self.getAnalyzer("PlotExecution").lines.setSplitting(splitThres=splitThres)

            self.plotExecution.title("Execution Time")

        if customRegexp:
            self.addAnalyzer("Custom",RegExpTimeLineLineAnalyzer("Custom",customRegexp))
            self.plotCustom=GnuplotTimelines(self.getAnalyzer("Custom").lines,persist=persist,with="lines")
            self.plotCustom.title("Custom")
           
    def timeHandle(self):
        if self.plot:
            self.plot.redo()
        if self.plotCont:
            self.plotCont.redo()
        if self.plotBound:
            self.plotBound.redo()
        if self.plotIter:
            self.plotIter.redo()
        if self.plotCourant:
            self.plotCourant.redo()
        if self.plotExecution:
            self.plotExecution.redo()
        if self.plotCustom:
            self.plotCustom.redo()

    def stopHandle(self):
        self.timeHandle()
        
        
class GnuplotRunner(GnuplotCommon,BasicRunner):
    def __init__(self,argv=None,smallestFreq=0.,persist=None,plotLinear=True,plotCont=True,plotBound=True,plotIterations=False,plotCourant=False,plotExecution=False,customRegexp=None):
        """@param smallestFreq: smallest Frequency of output
        @param persist: Gnuplot window persistst after run"""
        BasicRunner.__init__(self,argv=argv)
        GnuplotCommon.__init__(self,"Gnuplotting",smallestFreq=smallestFreq,persist=persist,plotLinear=plotLinear,plotCont=plotCont,plotBound=plotBound,plotIterations=plotIterations,plotCourant=plotCourant,plotExecution=plotExecution,customRegexp=customRegexp)

class GnuplotWatcher(GnuplotCommon,BasicWatcher):
    def __init__(self,logfile,smallestFreq=0.,persist=None,silent=False,tailLength=1000,sleep=0.1,plotLinear=True,plotCont=True,plotBound=True,plotIterations=False,plotCourant=False,plotExecution=False,customRegexp=None):
        """@param smallestFreq: smallest Frequency of output
        @param persist: Gnuplot window persistst after run"""
        BasicWatcher.__init__(self,logfile,silent=silent,tailLength=tailLength,sleep=sleep)
        GnuplotCommon.__init__(self,logfile,smallestFreq=smallestFreq,persist=persist,plotLinear=plotLinear,plotCont=plotCont,plotBound=plotBound,plotIterations=plotIterations,plotCourant=plotCourant,plotExecution=plotExecution,customRegexp=customRegexp)

    def startHandle(self):
        self.bakFreq=self.freq
        self.freq=3600
        
    def tailingHandle(self):
        self.freq=self.bakFreq
        self.oldtime=0
        
        
